"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMultiCompiler = exports.createCompiler = exports.rspack = void 0;
const config_1 = require("./config");
const compiler_1 = require("./compiler");
const util_1 = __importDefault(require("util"));
const rspackOptionsApply_1 = require("./rspackOptionsApply");
const NodeEnvironmentPlugin_1 = __importDefault(require("./node/NodeEnvironmentPlugin"));
const multiCompiler_1 = require("./multiCompiler");
const assert_1 = __importDefault(require("assert"));
const util_2 = require("./util");
function createMultiCompiler(options) {
    const compilers = options.map(option => {
        const compiler = createCompiler(option);
        /**
         * Missing features: WebpackOptionsApply
         * `compiler.name` should be set by WebpackOptionsApply.
         */
        compiler.name = option.name;
        return compiler;
    });
    const compiler = new multiCompiler_1.MultiCompiler(compilers, options);
    for (const childCompiler of compilers) {
        if (childCompiler.options.dependencies) {
            compiler.setDependencies(childCompiler, childCompiler.options.dependencies);
        }
    }
    return compiler;
}
exports.createMultiCompiler = createMultiCompiler;
function createCompiler(userOptions) {
    // console.log("user:", userOptions);
    const options = (0, config_1.getNormalizedRspackOptions)(userOptions);
    (0, config_1.applyRspackOptionsBaseDefaults)(options);
    (0, assert_1.default)(!(0, util_2.isNil)(options.context));
    const compiler = new compiler_1.Compiler(options.context, options);
    new NodeEnvironmentPlugin_1.default({
        infrastructureLogging: options.infrastructureLogging
    }).apply(compiler);
    const logger = compiler.getInfrastructureLogger("config");
    logger.debug("RawOptions:", util_1.default.inspect(userOptions, { colors: true, depth: null }));
    if (Array.isArray(options.plugins)) {
        for (const plugin of options.plugins) {
            if (typeof plugin === "function") {
                plugin.call(compiler, compiler);
            }
            else {
                plugin.apply(compiler);
            }
        }
    }
    (0, config_1.applyRspackOptionsDefaults)(compiler.options);
    logger.debug("NormalizedOptions:", util_1.default.inspect(compiler.options, { colors: true, depth: null }));
    compiler.hooks.environment.call();
    compiler.hooks.afterEnvironment.call();
    new rspackOptionsApply_1.RspackOptionsApply().process(compiler.options, compiler);
    compiler.hooks.initialize.call();
    return compiler;
}
exports.createCompiler = createCompiler;
function rspack(options, callback) {
    let compiler;
    if (Array.isArray(options)) {
        compiler = createMultiCompiler(options);
    }
    else {
        compiler = createCompiler(options);
    }
    if (callback) {
        compiler.run(callback);
        return compiler;
    }
    else {
        return compiler;
    }
}
exports.rspack = rspack;
//# sourceMappingURL=rspack.js.map